@@ -5,46 +5,52 @@ from robot.api.deco import keyword
 
 
 @keyword(name="Create Dataframe from file")
-def create_dataframe_from_file(file_path, delimiter=',', widths=None, has_header=True, skiprows=None, skipfooter=None):
+def create_dataframe_from_file(file_path, delimiter=',', has_header=True, width=None, encoding='ISO-8859-1', on_bad_lines='warn', skiprows=0, skipfooter=0):
     """
-    Creates a Pandas DataFrame from a CSV, PSV, or fixed width file.
+    Creates a Pandas DataFrame from a CSV, PSV, or fixed-width file.
 
     Parameters:
         file_path (str): The path to the input file.
         delimiter (str): The delimiter character used in the input file. Default is ','.
-        widths (list of int): The field widths for each column in the input file. If None, calculate the field widths. Default is None.
         has_header (bool): Whether the input file has headers. Default is True.
-        skiprows (list of int): Line numbers to skip at the start of the file.
-        skipfooter (int): Number of lines to skip at the end of the file.
+        width (list or tuple): A list or tuple of integers specifying the width of each fixed-width field.
+                              Required when reading a fixed-width file. Default is None.
+        encoding (str): The encoding of the input file. Default is 'ISO-8859-1'.
+        on_bad_lines (str): What to do with bad lines encountered in the input file.
+                            Valid values are 'raise', 'warn', and 'skip'. Default is 'warn'.
+        skiprows (int or list-like): Line numbers to skip (0-indexed) or number of lines to skip (int) at the start of the file. Default is 0.
+        skipfooter (int): Number of lines to skip at the end of the file. Default is 0.
 
     Returns:
         pandas.DataFrame: The DataFrame created from the input file.
 
     Examples:
     | Create Dataframe from file | /path/to/file.csv |
-    | Create Dataframe from file | /path/to/file.psv | delimiter='|', skiprows=[0], skipfooter=1 |
-    | Create Dataframe from file | /path/to/file.xlsx | skiprows=[0], skipfooter=2 |
+    | Create Dataframe from file | /path/to/file.psv | delimiter='|', has_header=False |
+    | Create Dataframe from file | /path/to/file.xlsx | has_header=False |
+    | Create Dataframe from file | /path/to/file.fwf | width=[10, 20, 30] |
+    | Create Dataframe from file | /path/to/file.csv | encoding='utf-8', on_bad_lines='raise' |
 
     """
     # Determine the file type based on the file extension
     file_ext = file_path.split('.')[-1].lower()
     if file_ext == 'csv':
         # Log info message
-        logger.info(f"Step 1: Reading CSV file '{file_path}' with delimiter '{delimiter}'")
+        logger.info(f"Step 1: Reading CSV file '{file_path}' with delimiter '{delimiter}' and encoding '{encoding}'")
         # Read CSV file into a DataFrame
         if has_header:
-            df = pd.read_csv(file_path, delimiter=delimiter, skiprows=skiprows, skipfooter=skipfooter)
+            df = pd.read_csv(file_path, delimiter=delimiter, encoding=encoding, on_bad_lines=on_bad_lines, skiprows=skiprows, skipfooter=skipfooter)
         else:
-            df = pd.read_csv(file_path, delimiter=delimiter, header=None, skiprows=skiprows, skipfooter=skipfooter)
+            df = pd.read_csv(file_path, delimiter=delimiter, header=None, encoding=encoding, on_bad_lines=on_bad_lines, skiprows=skiprows, skipfooter=skipfooter)
     elif file_ext == 'psv':
         # Log info message
         logger.info(f"Step 1: Reading PSV file '{file_path}' with delimiter '|'")
         # Read PSV file into a DataFrame
         if has_header:
-            df = pd.read_csv(file_path, delimiter='|', skiprows=skiprows, skipfooter=skipfooter)
+            df = pd.read_csv(file_path, delimiter='|', encoding=encoding, on_bad_lines=on_bad_lines, skiprows=skiprows, skipfooter=skipfooter)
         else:
-            df = pd.read_csv(file_path, delimiter='|', header=None, skiprows=skiprows, skipfooter=skipfooter)
-    elif file_ext == 'xlsx' or file_ext == 'xls':
+            df = pd.read_csv(file_path, delimiter='|', header=None, encoding=encoding, on_bad_lines=on_bad_lines, skiprows=skiprows, skipfooter=skipfooter)
+    elif file_ext == 'xlsx':
         # Log info message
         logger.info(f"Step 1: Reading Excel file '{file_path}'")
         # Read Excel file into a DataFrame
@@ -52,91 +58,23 @@ def create_dataframe_from_file(file_path, delimiter=',', widths=None, has_header
             df = pd.read_excel(file_path, skiprows=skiprows, skipfooter=skipfooter)
         else:
             df = pd.read_excel(file_path, header=None, skiprows=skiprows, skipfooter=skipfooter)
+    elif file_ext == 'fwf':
+        # Log info message
+        logger.info(f"Step 1: Reading fixed-width file '{file_path}' with width {width}")
+        # Read fixed-width file into a DataFrame
+        df = pd.read_fwf(file_path, widths=width, header=None, encoding=encoding, on_bad_lines=on_bad_lines,
+                         skiprows=skiprows, skipfooter=skipfooter)
     else:
-        raise ValueError("Unsupported file type.")
-
-    # Log success message
-    logger.info(f"Step 2: DataFrame created with shape {df.shape}")
+        # Log error message and raise exception
+        logger.error(
+            f"Error: Unsupported file type '{file_ext}'. Supported file types are 'csv', 'psv', 'xlsx', and 'fwf'.")
+        raise Exception(f"Unsupported file type '{file_ext}'")
 
+    # Log info message
+    logger.info(f"Step 2: DataFrame created with {df.shape[0]} rows and {df.shape[1]} columns")
+    # Return the DataFrame
     return df
 
-# @keyword(name="Create Dataframe from file")
-# def create_dataframe_from_file(file_path, delimiter=',', widths=None, has_header=True, skiprows=None, skipfooter=None):
-#     """
-#     Creates a Pandas DataFrame from a CSV, PSV, or fixed width file.
-#
-#     Parameters:
-#         file_path (str): The path to the input file.
-#         delimiter (str): The delimiter character used in the input file. Default is ','.
-#         widths (list of int): The field widths for each column in the input file. If None, calculate the field widths. Default is None.
-#         has_header (bool): Whether the input file has headers. Default is True.
-#         skiprows (list of int): Line numbers to skip at the start of the file.
-#         skipfooter (int): Number of lines to skip at the end of the file.
-#
-#     Returns:
-#         pandas.DataFrame: The DataFrame created from the input file.
-#
-#     Examples:
-#     | Create Dataframe from file | /path/to/file.csv |
-#     | Create Dataframe from file | /path/to/file.psv | delimiter='|' |
-#     | Create Dataframe from file | /path/to/file.txt | widths=[10, 5, 8, 10], has_header=False, skiprows=[0, 1], skipfooter=1 |
-#     | Create Dataframe from file | /path/to/file.dat | delimiter=' ' | widths=None, skipfooter=2 |
-#     | Create Dataframe from file | /path/to/file.xlsx |
-#
-#     """
-#     # Determine the file type based on the file extension
-#     file_ext = file_path.split('.')[-1].lower()
-#     if file_ext == 'csv':
-#         # Log info message
-#         logger.info(f"Step 1: Reading CSV file '{file_path}' with delimiter '{delimiter}'")
-#         # Read CSV file into a DataFrame
-#         if has_header:
-#             df = pd.read_csv(file_path, delimiter=delimiter)
-#         else:
-#             df = pd.read_csv(file_path, delimiter=delimiter, header=None)
-#     elif file_ext == 'psv':
-#         # Log info message
-#         logger.info(f"Step 1: Reading PSV file '{file_path}' with delimiter '|'")
-#         # Read PSV file into a DataFrame
-#         if has_header:
-#             df = pd.read_csv(file_path, delimiter='|', skiprows=skiprows, skipfooter=skipfooter)
-#         else:
-#             df = pd.read_csv(file_path, delimiter='|', header=None, skiprows=skiprows, skipfooter=skipfooter)
-#     elif file_ext == 'txt' or file_ext == 'dat':
-#         if widths is None:
-#             # Calculate field widths
-#             logger.info(f"Step 1: Calculating field widths for fixed width file '{file_path}'")
-#             with open(file_path, 'r') as f:
-#                 # Read the first line of the file to determine the number of columns
-#                 num_cols = len(f.readline().strip().split(delimiter))
-#                 # Calculate the width of each column by dividing the width of the file by the number of columns
-#                 width = int(f.tell() / num_cols)
-#                 # Set the widths parameter to a list of equal width integers
-#                 widths = [width] * num_cols
-#                 logger.info(f"Calculated widths: {widths}")
-#         else:
-#             logger.info(f"Step 1: Reading fixed width file '{file_path}' with widths {widths}")
-#         # Read fixed width file into a DataFrame
-#         if has_header:
-#             df = pd.read_fwf(file_path, widths=widths, skiprows=[0], skipfooter=1)
-#         else:
-#             df = pd.read_fwf(file_path, widths=widths, header=None, skiprows=[0], skipfooter=1)
-#     elif file_ext == 'xlsx' or file_ext == 'xls':
-#         # Log info message
-#         logger.info(f"Step 1: Reading Excel file '{file_path}'")
-#         # Read Excel file into a DataFrame
-#         if has_header:
-#             df = pd.read_excel(file_path, skiprows=skiprows, skipfooter=skipfooter)
-#         else:
-#             df = pd.read_excel(file_path, header=None, skiprows=skiprows, skipfooter=skipfooter)
-#     else:
-#         raise ValueError("Unsupported file type.")
-#
-#     # Log success message
-#     logger.info(f"Step 2: DataFrame created with shape {df.shape}")
-#
-#     return df
-
 
 @keyword("Write DataFrame to CSV file")
 def write_df_to_csv(df_to_write: pd.DataFrame, file_path: str, file_name: str, index: bool = False):
@@ -353,26 +291,61 @@ def df_diff(actual_file_path: str, expected_file_path: str, actual_file_name: st
 #     Compare two DataFrames and show differences  ${actual_file_path}  ${expected_file_path}  ${actual_file_name}  ${expected_file_name}  ${file_format}  ${key_columns}  ${ignore_columns}
 
 # *** Settings ***
-# Library    Pandas
-# Library    OperatingSystem
-
+# Library  RFPandasUtils.py
+#
 # *** Test Cases ***
-# Test Create DataFrame from File
-#     [Documentation]    Test the 'Create Dataframe from file' keyword
-#     [Tags]    create_dataframe_from_file
+#Test Write DataFrame to CSV file
+#    ${df}=  Create Dataframe  1,2,3\n4,5,6\n7,8,9
+#    Write DataFrame to CSV file  ${df}  ./  test.csv
+#    ${written_df}=  Read CSV  ./test.csv
+#    Should Be True  ${df}.equals(${written_df})
 #
-#     ${df1}    Create Dataframe from file    ${CURDIR}/data/test.csv
-#     Should Be Equal As Integers    ${df1.shape[0]}    5
-#     Should Be Equal As Integers    ${df1.shape[1]}    3
+#Test Write DataFrame to PSV file
+#    ${df}=  Create Dataframe  1,2,3\n4,5,6\n7,8,9
+#    Write DataFrame to PSV file  ${df}  ./  test.psv
+#    ${written_df}=  Read CSV  ./test.psv  delimiter=|
+#    Should Be True  ${df}.equals(${written_df})
 #
-#     ${df2}    Create Dataframe from file    ${CURDIR}/data/test.psv    delimiter='|'
-#     Should Be Equal As Integers    ${df2.shape[0]}    5
-#     Should Be Equal As Integers    ${df2.shape[1]}    3
+#Test Compare two DataFrames and show differences
+#    ${actual_file_path}=  Set Variable  ./test_data/
+#    ${expected_file_path}=  Set Variable  ./baseline_data/
+#    ${actual_file_name}=  Set Variable  actual_file
+#    ${expected_file_name}=  Set Variable  expected_file
+#    ${file_format}=  Set Variable  csv
+#    ${key_columns}=  Create List  Key_Column1  Key_Column2
+#    ${ignore_columns}=  Create List  Ignore_Column1  Ignore_Column2
+#    ${expected_df}=  Read CSV  ${expected_file_path}${expected_file_name}.${file_format}
+#    ${actual_df}=  Read CSV  ${actual_file_path}${actual_file_name}.${file_format}
+#    Compare two DataFrames and show differences  ${actual_file_path}  ${expected_file_path}  ${actual_file_name}  ${expected_file_name}  ${file_format}  ${key_columns}  ${ignore_columns}
 #
-#     ${df3}    Create Dataframe from file    ${CURDIR}/data/test.txt    widths=[10, 5, 8], has_header=False, skiprows=[0, 1], skipfooter=1
-#     Should Be Equal As Integers    ${df3.shape[0]}    3
-#     Should Be Equal As Integers    ${df3.shape[1]}    3
+
+#*** Test Cases ***
+# Read CSV file with header
+#     [Documentation]    Read data from a CSV file with header
+#     ${dataframe}=    Create Dataframe from file    ../tests/5m Sales Records.csv
+#     Log    ${dataframe}
 #
-#     ${df4}    Create Dataframe from file    ${CURDIR}/data/test.xlsx
-#     Should Be Equal As Integers    ${df4.shape[0]}    5
-#     Should Be Equal As Integers    ${df4.shape[1]}    3
+# Read CSV file without header
+#     [Documentation]    Read data from a CSV file without header
+#     ${dataframe}=    Create Dataframe from file    ../tests/5m Sales Records_without_header.csv    has_header=False
+#     Log    ${dataframe}
+#
+# Read Excel file with header
+#     [Documentation]    Read data from an Excel file with header
+#     ${dataframe}=    Create Dataframe from file    ../tests/5m Sales Records.xlsx
+#     Log    ${dataframe}
+#
+# Read Excel file without header
+#     [Documentation]    Read data from an Excel file without header
+#     ${dataframe}=    Create Dataframe from file    ../tests/5m Sales Records_without_header.xlsx    has_header=False
+#     Log    ${dataframe}
+#
+# Read PSV file with header
+#     [Documentation]    Read data from a PSV file with header
+#     ${dataframe}=    Create Dataframe from file    ../tests/10000 Sales Records.psv    delimiter='|'
+#     Log    ${dataframe}
+#
+# Read PSV file without header
+#     [Documentation]    Read data from a PSV file without header
+#     ${dataframe}=    Create Dataframe from file    ../tests/10000 Sales Records_without_header.psv    delimiter='|'    has_header=False
+#     Log    ${dataframe}
